@@ -34,16 +34,26 @@ class CommandNeedsNonNullInteger(Exception):
     pass
 
 
-class CommandNeedsBool(Exception):
+class InvalidArgumentForCommand(Exception):
+    def __init__(self, command, value, possible_values):
+        super().__init__()
+        self.possible_values = possible_values
+        self.command = command
+        self.value = value
+
     pass
 
 
+INTERACTIVE_BANNER = \
+    f'''Decision Optimization in WML Interactive, version {dowml.VERSION}.
+Submit and manage Decision Optimization models interactively.
+(c) Copyright IBM Corp. 2021'''
+
+
 class DOWMLInteractive(Cmd):
     prompt = 'dowml> '
     intro = (f'''
-Decision Optimization in WML Interactive, version {dowml.VERSION}.
-Submit and manage Decision Optimization models interactively.
-(c) Copyright IBM Corp. 2021
+{INTERACTIVE_BANNER}
 
 Type ? for a list of commands.
 
@@ -88,32 +98,41 @@ job id, but none is specified, the last one is used.
         """Exit this program."""
         return True
 
+    def print_known_types(self):
+        known_types = ', '.join(self.lib.MODEL_TYPES)
+        print(f'Known types: {known_types}.')
+
     def do_type(self, model_type):
         """type [type-of-model]
-Prints current model type (if no argument), or sets the model type."""
-        known_types = ', '.join(self.lib.MODEL_TYPES)
+Prints current model type and known types (if no argument), or sets the model type."""
         if not model_type:
-            print(f'Current model type: {self.lib.model_type}. Known types: {known_types}')
+            print(f'Current model type: {self.lib.model_type}.')
+            self.print_known_types()
             # Let's make sure we don't set the model_type to None, but
             # return immediately
             return
         if model_type not in self.lib.MODEL_TYPES:
-            print(f'Warning: unknown model type \'{model_type}\'. Known types: {known_types}')
+            print(f'Warning: unknown model type \'{model_type}\'.')
+            self.print_known_types()
         # We set the type nevertheless: this code may not be up-to-date
         self.lib.model_type = model_type
 
+    def print_known_sizes(self):
+        known_sizes = ', '.join(self.lib.TSHIRT_SIZES)
+        print(f'Known sizes: {known_sizes}.')
+
     def do_size(self, tshirt_size):
         """size [size-of-deployment]
-Prints current deployment size (if no argument), or sets the deployment size."""
-        sizes = self.lib.TSHIRT_SIZES
-        known_sizes = ', '.join(sizes)
+Prints current deployment size and known sizes (if no argument), or sets the deployment size."""
         if not tshirt_size:
-            print(f'Current size: {self.lib.tshirt_size}. Known sizes: {known_sizes}')
+            print(f'Current size: {self.lib.tshirt_size}.')
+            self.print_known_sizes()
             # Let's make sure we don't set the tshirt_size to None, but
             # return immediately
             return
-        if tshirt_size not in sizes:
-            print(f'Warning: unknown tee-shirt size \'{tshirt_size}\'. Known sizes: {known_sizes}')
+        if tshirt_size not in self.lib.TSHIRT_SIZES:
+            print(f'Warning: unknown tee-shirt size \'{tshirt_size}\'.')
+            self.print_known_sizes()
         # We set the size nevertheless: this code may not be up-to-date
         self.lib.tshirt_size = tshirt_size
 
@@ -138,13 +157,13 @@ Prints current time limit (if no argument), or sets the time limit (in seconds).
 Prints default DO version (if no argument) and available ones, or sets the DO version to the specified string."""
         if not version:
             print(f'Decision Optimization version: {self.lib.do_version}')
-            str = ', '.join(self.lib.get_available_versions())
-            print(f'Available versions: {str}')
+            versions = ', '.join(self.lib.get_available_versions())
+            print(f'Available versions: {versions}')
             return
         self.lib.do_version = version
 
     def do_inline(self, arg):
-        """inline [yes|no]
+        """inline [yes|no]   --  DEPRECATED: use 'inputs' instead
 Prints whether jobs are created with inline data (if no argument), or sets the
 flag ('yes' or 'no').
 
@@ -153,7 +172,8 @@ platform for their inputs, rather than uploading the content of files as part of
 each job payload.  If a file name (stripped from its path) matches the name of
 an existing data asset, the content of the file is not uploaded (actually, the
 file may not even exist locally) and the content of that data asset is used
-instead.
+instead. If a file name is prefixed with a '+', an existing data asset will be
+replaced with the content of the local file.
 
 In 'inline yes' mode, the content of the specified files is always uploaded to
 the platform as part of the job payload, and not stored as a data asset."""
@@ -162,11 +182,55 @@ the platform as part of the job payload, and not stored as a data asset."""
             print(f'inline: {flag}')
             return
         if arg == 'yes':
-            self.lib.inline = True
+            self.lib.inputs = 'inline'
         elif arg == 'no':
-            self.lib.inline = False
+            self.lib.inputs = 'assets'
         else:
-            raise CommandNeedsBool
+            raise InvalidArgumentForCommand('inline', arg, ['yes', 'no'])
+
+    def do_inputs(self, arg):
+        """inputs [inline|assets]
+Displays (if no argument) or sets (if given an argument) the type of inputs
+that jobs will use.  Possible values are 'inline' and 'assets'.
+
+In 'inputs inline' mode, jobs use inline input data: the files that constitute
+the job are included in the job creation request.
+
+In 'inputs assets' mode, which is the default, jobs use data assets stored on the
+platform for their inputs, rather than uploading the content of files as part of
+each job payload.  If a file name (stripped from its path) matches the name of
+an existing data asset, the content of the file is not uploaded (actually, it's
+ok if the file does not even exist locally) and the content of that data asset is
+used instead. If a file name is prefixed with a '+', the file is always uploaded
+and an existing data asset is replaced with the content of the local file."""
+        if not arg:
+            print(f'inputs: {self.lib.inputs}')
+            return
+        allowed_values = ['inline', 'assets']
+        if arg in allowed_values:
+            self.lib.inputs = arg
+        else:
+            raise InvalidArgumentForCommand('inputs', arg, allowed_values)
+
+    def do_outputs(self, arg):
+        """outputs [inline|assets]
+Displays (if no argument) or sets (if given an argument) the type of outputs
+that jobs will use.  Possible values are 'inline' and 'assets'.
+
+In 'outputs inline' mode, which is the default, jobs use inline output data: the
+results of the job are included in the job details. This is fine for most jobs.
+
+In 'outputs assets' mode, the job outputs are stored as data assets in the job's
+deployment space.  Their names start with the id of the job, followed by the
+file's name. This is useful for jobs that create very large outputs."""
+        if not arg:
+            print(f'outputs: {self.lib.outputs}')
+            return
+        allowed_values = ['inline', 'assets']
+        if arg in allowed_values:
+            self.lib.outputs = arg
+        else:
+            raise InvalidArgumentForCommand('outputs', arg, allowed_values)
 
     def do_solve(self, paths):
         """solve file1 [file2 ... [filen]]
@@ -206,7 +270,8 @@ job is either a job number or a job id. Uses current job if not specified."""
 Lists all the jobs in the space.
 Current job, if any, is indicated with an arrow."""
         jobs = self._cache_jobs()
-        print('     #  status      id                                    creation date        type     ver.   size  inputs')
+        print('     #  status      id                                    '
+              'creation date        type     ver.   size  inputs')
         for i, j in enumerate(jobs, start=1):
             # Prepare list of input files
             names = ', '.join(j.names)
@@ -214,7 +279,8 @@ Current job, if any, is indicated with an arrow."""
             mark = '   '
             if j.id == self.last_job_id:
                 mark = '=> '
-            print(f'{mark}{i:>3}: {j.status:<10}  {j.id}  {j.created}  {j.type:<7}  {j.version:<5}  {j.size:<4}  {names}')
+            print(f'{mark}{i:>3}: {j.status:<10}  {j.id}  '
+                  f'{j.created}  {j.type:<7}  {j.version:<5}  {j.size:<4}  {names}')
 
     def do_log(self, job_id):
         """log [job]
@@ -230,14 +296,17 @@ Downloads all the outputs of a job, as well as the details/status of the job.
 job is either a job number or a job id. Uses current job if not specified."""
         job_id = self._get_and_remember_job_id(job_id)
         details = self.lib.get_job_details(job_id, with_contents='full')
-        outputs = self.lib.get_output(details, tabular_as_csv=True)
-        for name in outputs:
-            self.save_content(job_id, name, outputs[name])
+        self._output_regular_files(job_id, details)
         # We don't want to store all the outputs in the details themselves
         self.lib.filter_large_chunks_from_details(details)
         details = pprint.pformat(details)
         self.save_content(job_id, 'details.json', details, text=True)
 
+    def _output_regular_files(self, job_id, details):
+        outputs = self.lib.get_output(details, tabular_as_csv=True)
+        for name in outputs:
+            self.save_content(job_id, name, outputs[name])
+
     @staticmethod
     def save_content(job_id, name, content, text=False):
         try:
@@ -267,15 +336,26 @@ job is either a job number or a job id. Uses current job if not specified."""
         printer(details, indent=4, width=120)
 
     def do_delete(self, job_id):
-        """delete [job|*]
-Deletes the job specified. Use '*' to delete all the jobs.
-job is either a job number or a job id. Uses current job if not specified."""
+        """delete [job|*|n-m]
+Deletes the job specified. 'job' is either a job number or a job id.
+Use '*' to delete all the jobs. A range of jobs, specified by their numbers,
+can be deleted using 'n-m'.
+Uses current job if not specified."""
         if job_id == '*':
             # Update the list of jobs before trying to delete them
             self._cache_jobs()
             while self.jobs:
                 job = self.jobs[0]
                 self.delete_one_job(job)
+        elif m := re.fullmatch('(\\d+)-(\\d+)', job_id):
+            n1 = int(m.group(1))
+            n2 = int(m.group(2))
+            # We must gather the list of all the ids of jobs we want to delete
+            # right from the start, because deleting each job will update the
+            # list while iterating
+            ids = self.jobs[n1-1:n2]
+            for i in ids:
+                self.delete_one_job(i)
         else:
             self.delete_one_job(job_id)
 
@@ -305,7 +385,8 @@ job is either a job number or a job id. Uses current job if not specified."""
         job_id = self._get_and_remember_job_id(job_id)
         self.lib.delete_job(job_id, False)
 
-    def do_shell(self, command):
+    @staticmethod
+    def do_shell(command):
         """shell command
 Runs the specified command in a shell."""
         os.system(command)
@@ -315,50 +396,50 @@ Runs the specified command in a shell."""
 
 
 def main_loop(wml_cred_file, space_id, commands, prompt_at_the_end):
-    try:
-        dowml = DOWMLInteractive(wml_cred_file, space_id)
-        # By default, we want to run the command loop
-        loop = True
-        for c in commands:
-            if loop:
-                # This is the first command we carry out. Let's print intro
-                print(dowml.intro, end='')
-                # And make sure we won't print it again
-                dowml.intro = ''
-            # We run the command loop iff this was asked for
-            loop = prompt_at_the_end
-            print(f'{dowml.prompt}{c}')
-            dowml.onecmd(c)
-        while loop:
-            # Generally speaking, one command loop is all we need, and we
-            # should not run another, except in case of errors
-            loop = False
-            try:
-                dowml.cmdloop()
-            except KeyboardInterrupt:
-                # The user interrupted. That's perfectly fine...
-                loop = True
-            except ApiRequestFailure:
-                # This happens when an invalid job id is specified. We want
-                # to keep running.
-                loop = True
-            except requests.exceptions.ConnectionError as e:
-                print(e)
-                loop = True
-            except CommandNeedsJobID:
-                print(f'This command requires a jod id or number, but you '
-                      f'didn\'t specify one.  And there is no current job either.')
-                loop = True
-            except NoCredentialsToCreateSpace as e:
-                print(e)
-                # There's nothing the user could do at this point: no need to loop
-            finally:
-                # But let's not print again the starting banner
-                dowml.intro = ''
-    except InvalidCredentials:
-        print(f'\nERROR: credentials not found!\n')
-        parser.print_help()
-        sys.exit(1)
+    instance = DOWMLInteractive(wml_cred_file, space_id)
+    # By default, we want to run the command loop
+    loop = True
+    for c in commands:
+        if loop:
+            # This is the first command we carry out. Let's print intro
+            print(instance.intro, end='')
+            # And make sure we won't print it again
+            instance.intro = ''
+        # We run the command loop iff this was asked for
+        loop = prompt_at_the_end
+        print(f'{instance.prompt}{c}')
+        instance.onecmd(c)
+    while loop:
+        # Generally speaking, one command loop is all we need, and we
+        # should not run another, except in case of errors
+        loop = False
+        try:
+            instance.cmdloop()
+        except KeyboardInterrupt:
+            # The user interrupted. That's perfectly fine...
+            loop = True
+        except ApiRequestFailure:
+            # This happens when an invalid job id is specified. We want
+            # to keep running.
+            loop = True
+        except requests.exceptions.ConnectionError as e:
+            print(e)
+            loop = True
+        except CommandNeedsJobID:
+            print('This command requires a jod id or number, but you '
+                  'didn\'t specify one.  And there is no current job either.')
+            loop = True
+        except InvalidArgumentForCommand as e:
+            print(f'Invalid argument \'{e.value}\' for command \'{e.command}\'. '
+                  f'Possible values are {e.possible_values}. '
+                  f'Please type \'help {e.command}\' to learn more.')
+            loop = True
+        except NoCredentialsToCreateSpace as e:
+            print(e)
+            # There's nothing the user could do at this point: no need to loop
+        finally:
+            # But let's not print again the starting banner
+            instance.intro = ''
 
 
 # We will mock the 'requests' function that's used by APIClient
@@ -379,24 +460,28 @@ def mocked_requests_session_send(*arguments, **kwargs):
 
 
 def interactive():
-    parser = argparse.ArgumentParser(description='Interactive program for DO on WML')
+    parser = argparse.ArgumentParser(description=INTERACTIVE_BANNER,
+                                     formatter_class=argparse.RawDescriptionHelpFormatter)
     parser.add_argument('--wml-cred-file', '-w', default=None,
                         help=f'Name of the file from which to read WML '
-                             f'credentials. If not specified, credentials ' 
+                             f'credentials. If not specified, credentials '
                              f'are read from environment variable '
-                             f'${_CredentialsProvider.ENVIRONMENT_VARIABLE_NAME}.')
+                             f'${_CredentialsProvider.ENVIRONMENT_VARIABLE_NAME}. If '
+                             f'no such variable exists, but variable '
+                             f'${_CredentialsProvider.ENVIRONMENT_VARIABLE_NAME_FILE} '
+                             f'exists, tries to read that file.')
     parser.add_argument('--verbose', '-v', action='count', default=0,
-                        help=f'Verbose mode.  Causes the program to print debugging '
-                             f'messages about its progress.  Multiple -v options '
-                             f'increase the verbosity.  The maximum is 4.')
+                        help='Verbose mode.  Causes the program to print debugging '
+                             'messages about its progress.  Multiple -v options '
+                             'increase the verbosity.  The maximum is 4.')
     parser.add_argument('--commands', '-c', nargs='*', default=[],
-                        help=f'Carries out the specified commands.  Each command '
-                             f'is executed as if it had been specified at the prompt. '
-                             f'The program stops after last command, unless --input '
-                             f'is used.')
+                        help='Carries out the specified commands.  Each command '
+                             'is executed as if it had been specified at the prompt. '
+                             'The program stops after last command, unless --input '
+                             'is used.')
     parser.add_argument('--input', '-i', action='store_true',
-                        help=f'Prompts for new input commands even if some commands '
-                             f'have been specified as arguments using --commands.')
+                        help='Prompts for new input commands even if some commands '
+                             'have been specified as arguments using --commands.')
     parser.add_argument('--space', '-s', default=None,
                         help=f'Id of the space to connect to. Takes precedence over '
                              f'the one specified in the credentials under the '
@@ -425,8 +510,13 @@ def interactive():
         # logging.getLogger('ibm_botocore').setLevel(logging.DEBUG)
         # logging.getLogger('ibm_boto3').setLevel(logging.DEBUG)
 
-    main_loop(args.wml_cred_file, args.space, args.commands, args.input)
+    try:
+        main_loop(args.wml_cred_file, args.space, args.commands, args.input)
+    except InvalidCredentials:
+        print('\nERROR: credentials not found!\n')
+        parser.print_help()
+        sys.exit(1)
 
 
 if __name__ == '__main__':
-    interactive()
\ No newline at end of file
+    interactive()
